function [vNeffMin, vNeffMax, NEFF_FULL, ERR_FULL] = find_modes_rec(minNeff...
    , maxNeff, minErr, maxErr, fun, sm, bPlot, absMinErr, hAx, hWaitbar, bFirstTime)
global NEFF_FULL ERR_FULL NMIN NMONO NDIVS prev_neff bSkip bCancel 
tol = 0.01/max(1,(sm-6));
if bFirstTime
    bFirstTime = 0;
    NEFF_FULL = [];
    ERR_FULL = [];
    NMONO = 0;
    NMIN = 0;
    NDIVS = 1;
    bSkip = 0;
    bCancel = 0;
end
if bCancel
    vNeffMin = NaN;
    vNeffMax = NaN;
    NEFF_FULL = NaN;
    ERR_FULL= NaN;
    return
end
if bSkip
    vNeffMin = [];
    vNeffMax = [];
    return
end

bInInterval = (NEFF_FULL>=minNeff) & (NEFF_FULL<=maxNeff);
vErr_full = ERR_FULL(bInInterval);
vNeff_full = NEFF_FULL(bInInterval);
if length(vNeff_full) < 3
    midNeff = (maxNeff+minNeff)/2;
    midErr = fun(midNeff);
    if minErr == 0
        minErr = fun(minNeff);
    end
    if maxErr == 0
        maxErr = fun(maxNeff);
    end
    [vNeff_full, vErr_full] = unique_add(vNeff_full, vErr_full, [minNeff midNeff maxNeff], [minErr midErr maxErr]);
    [NEFF_FULL, ERR_FULL] = unique_add(NEFF_FULL, ERR_FULL, vNeff_full, vErr_full);
    if bPlot
        plotnow
    end
end

nLen = 3;
if length(vNeff_full) < 3 % in the rare case that the interval has become too small
    vNeffMax = [];
    vNeffMin = [];
    NMONO = NMONO+1;
    return
end
bTestEdge = true;
while (isempty([local_maxima(vErr_full) local_maxima(1./vErr_full)]) )
    if bPlot
        plotnow
    end
    if bCancel
        vNeffMin = NaN;
        vNeffMax = NaN;
        NEFF_FULL = NaN;
        ERR_FULL= NaN;
        return
    end
    if bSkip
        vNeffMin = [];
        vNeffMax = [];
        return
    end
    if nLen >= sm
        vNeffMax = [];
        vNeffMin = [];
        NMONO = NMONO+1;
        return
    end
    newNeff = [];
    if bTestEdge
        bTestEdge = false;
        if vErr_full(1) < absMinErr % avoid doing this near max where there is noise
            newNeff = (maxNeff-minNeff)/sm/10 + minNeff;
        end
        if vErr_full(end) < absMinErr % avoid doing this near max where there is noise
            newNeff = [newNeff maxNeff - (maxNeff-minNeff)/sm/10];
        end
    else
        nLen = nLen+2;
        maxAlpha = 0;
        iMax = 2;
        for iNeff = 2:length(vNeff_full)-1
            neff = vNeff_full(iNeff-1:iNeff+1);
            err = vErr_full(iNeff-1:iNeff+1);
            [p,S,mu] = polyfit(neff, err, 1);
            if std(err) == 0
                alpha = inf;
            else
                alpha =(S.normr/2/std(err));
            end
            if maxAlpha < alpha
                iMax = iNeff;
                maxAlpha = alpha;
            end
        end
        neff = vNeff_full(iMax-1:iMax+1);
        newNeff = neff(1:2)+1/2*diff(neff);
    end
    newErr = fun(newNeff);
    [vNeff_full, vErr_full] = unique_add(vNeff_full, vErr_full, newNeff, newErr);
    [NEFF_FULL, ERR_FULL] = unique_add(NEFF_FULL, ERR_FULL, vNeff_full, vErr_full);
    if bPlot
        plotnow
    end
    if NMIN == 1
        sTitle = [num2str(NMIN) ' minimum found...'];
    else
        sTitle = [num2str(NMIN) ' minima found...'];
    end
    waitbar((NMONO+nLen/sm)/(NDIVS), hWaitbar, sTitle)
end
[dummy, iMin] = local_maxima(1./vErr_full);
if ~isempty(iMin) % a minimum has been found
    NMIN = NMIN + 1;
    NDIVS = NDIVS + 1;
    if NMIN == 1
        sTitle = [num2str(NMIN) ' minimum found...'];
    else
        sTitle = [num2str(NMIN) ' minima found...'];
    end
    minLocal = vNeff_full(iMin(1)-1);
    maxLocal = vNeff_full(iMin(1)+1);
    bMax = 0;
    minTol = (maxLocal-minLocal)*tol;
    [neff, err] = fminbnd(fun, minLocal, maxLocal, optimset('TolX', minTol, 'OutputFcn', @fine_search_cancel));
    if bCancel
        vNeffMin = NaN;
        vNeffMax = NaN;
        NEFF_FULL = NaN;
        ERR_FULL= NaN;
        return
    end
    if bSkip
        vNeffMin = [];
        vNeffMax = [];
        return
    end
    [dummy, iNeff] = min(abs(neff-NEFF_FULL));
    neffRight = NEFF_FULL(iNeff+1);
    neffLeft = NEFF_FULL(iNeff-1);
    [vRNeffMin, vRNeffMax] = find_modes_rec(neffRight, maxNeff, 0, maxErr, fun, sm, bPlot, absMinErr, hAx, hWaitbar, bFirstTime);
    [vLNeffMin, vLNeffMax] = find_modes_rec(minNeff, neffLeft, minErr, 0, fun, sm, bPlot, absMinErr, hAx, hWaitbar, bFirstTime);
    if bCancel
        vNeffMin = NaN;
        vNeffMax = NaN;
        NEFF_FULL = NaN;
        ERR_FULL= NaN;
        return
    end
    if bSkip
        vNeffMin = [];
        vNeffMax = [];
        return
    end
    vNeffMin = [vLNeffMin neff vRNeffMin];
    vNeffMax = [vLNeffMax vRNeffMax];
else % a maximum has been found
    NDIVS = NDIVS + 1;
    if NMIN == 1
        sTitle = [num2str(NMIN) ' minimum found...'];
    else
        sTitle = [num2str(NMIN) ' minima found...'];
    end
    [dummy, iMax] = local_maxima(vErr_full);
    minLocal = vNeff_full(iMax(1)-1);
    maxLocal = vNeff_full(iMax(1)+1);
    bMax = 1;
    maxTol = (maxLocal-minLocal)*tol*10;
    [neff, acc] = fminbnd(@(x) 1/fun(x), minLocal, maxLocal, optimset('TolX', maxTol, 'OutputFcn', @fine_search_cancel));
    if bCancel
        vNeffMin = NaN;
        vNeffMax = NaN;
        NEFF_FULL = NaN;
        ERR_FULL= NaN;
        return
    end
    if bSkip
        vNeffMin = [];
        vNeffMax = [];
        return
    end
    [dummy, iNeff] = min(abs(neff-NEFF_FULL));
    neffRight = NEFF_FULL(iNeff+1);
    neffLeft = NEFF_FULL(iNeff-1);
    [vRNeffMin, vRNeffMax] = find_modes_rec(neffRight, maxNeff, 0, maxErr, fun, sm, bPlot, absMinErr, hAx, hWaitbar, bFirstTime);
    [vLNeffMin, vLNeffMax] = find_modes_rec(minNeff, neffLeft, minErr, 0, fun, sm, bPlot, absMinErr, hAx, hWaitbar, bFirstTime);
    if bCancel
        vNeffMin = NaN;
        vNeffMax = NaN;
        NEFF_FULL = NaN;
        ERR_FULL= NaN;
        return
    end
    if bSkip
        vNeffMin = [];
        vNeffMax = [];
        return
    end
    vNeffMin = [vLNeffMin vRNeffMin];
    vNeffMax = [vLNeffMax neff vRNeffMax];
end

    function stop = fine_search_cancel(beta, val, state, varargin)
        if ~isempty(beta)
            if length(beta)==2
                beta = beta(1)+j*beta(2);
            end
            curr_neff = beta;
            if isempty(prev_neff)
                this_mode_prog = 1;
            else
                this_mode_prog = tol/(abs(curr_neff-prev_neff)+eps);
            end
            waitbar((NMONO+min(1, this_mode_prog))/NDIVS, hWaitbar, sTitle)
            prev_neff = curr_neff;
            curr_err = val.fval;
            if bMax
                curr_err = 1/curr_err;
            end
            [NEFF_FULL, ERR_FULL] = unique_add(NEFF_FULL, ERR_FULL, curr_neff, curr_err);
            if bPlot
                plotnow
            end
        end
        drawnow
        stop = bCancel | bSkip;
    end

    function plotnow()
        hold(hAx, 'off')
        plot(hAx, NEFF_FULL, (ERR_FULL), 'b', NEFF_FULL, (ERR_FULL), 'r.'), hold on
        axis(hAx, [min(NEFF_FULL), max(NEFF_FULL), 0, max(ERR_FULL)*1.1]);
        xlabel(hAx, '$\beta / k_0$')
        ylabel(hAx, '$\Delta E$')
        drawnow
    end
end