function [mZ, varargout] = imp_matrix(oGd, pos, k, beta, isImproper)
global EPSILON0 C ETA
if isreal(beta)
    isImproper = 0;
end
w = k*C;
sdArray = oGd.sdArray;
mZ = zeros(sum([pos.testing.n])*4, sum([pos.sources.n])*2);
mBNegate = logical(mZ);
if isfield(oGd, 'hGreenFun')
    hGreenFun = oGd.hGreenFun;
else
    hGreenFun = @green_fun;
end
% these vectors give the edge indices of each region in the matrix
vISources = [0 cumsum(2*[pos.sources.n])];
vITesting = [0 cumsum([pos.testing.n])];
indEz = size(mZ,1)/4;
indHt = 2*indEz;
indHz = 3*indEz;
for iSd = 1:oGd.nSd
    if pos.sources(iSd).n == 0
        continue
    end
    Er = sdArray(iSd).Er;
    if ischar(Er)
        Er = feval(Er, 2*pi/k);
    end
    Y = j*EPSILON0*Er*w;
    % testing points on the 'inside' curve
    curve = sdArray(iSd).insideCurve;
    if ~isempty(curve)
        % When the modes are leaky, the inside should be proper
        % for numerical reasons.
        kc = calc_kc(k, beta, Er, 0);
        iCurve = pos.testing(curve.iCurve).iCurve;
%        vGreen = pos.sources(iSd).vGreen;
%        pos.sources(iSd).vGreen = [1 zeros(1,10) 1];
        [TE_Et, TE_Ez, TE_Ht, TE_Hz, TM_Et, TM_Ez, TM_Ht, TM_Hz] = ...
            feval(hGreenFun, pos.testing(iCurve), pos.sources(iSd), Y, kc, beta, oGd);
%        pos.sources(iSd).vGreen = vGreen;
        rows = vITesting(iCurve)+1:vITesting(iCurve+1);
        cols = vISources(iSd)+1:vISources(iSd+1);
        mZ(rows, cols) =       [ETA*TE_Et   TM_Et];
        mZ(rows+indEz, cols) = [ETA*TE_Ez   TM_Ez];
        mZ(rows+indHt, cols) = [ETA^2*TE_Ht ETA*TM_Ht];
        mZ(rows+indHz, cols) = [ETA^2*TE_Hz ETA*TM_Hz];
        mNaN = isnan(mZ);
        if any(mNaN)
            new_cols = sum(~mNaN(1,:));
            mZ = reshape(mZ(~mNaN), rows, new_cols);
        end
    end
    % testing points on the 'outside' curves
    for iOutsideCurve = 1:length(sdArray(iSd).outsideCurveArray)
        curve = sdArray(iSd).outsideCurveArray(iOutsideCurve);
        if ~isempty(curve)
            if pos.testing(curve.iCurve).n == 0
                continue
            end
            kc = calc_kc(k, beta, Er, isImproper); 
            iCurve = pos.testing(curve.iCurve).iCurve;
            [TE_Et, TE_Ez, TE_Ht, TE_Hz, TM_Et, TM_Ez, TM_Ht, TM_Hz] = ...
                feval(hGreenFun, pos.testing(iCurve), pos.sources(iSd), Y, kc, beta, oGd);
            rows = vITesting(iCurve)+1:vITesting(iCurve+1);
            cols = vISources(iSd)+1:vISources(iSd+1);
            mZ(rows, cols) =       -[ETA*TE_Et   TM_Et];
            mZ(rows+indEz, cols) = -[ETA*TE_Ez   TM_Ez];
            mZ(rows+indHt, cols) = -[ETA^2*TE_Ht ETA*TM_Ht];
            mZ(rows+indHz, cols) = -[ETA^2*TE_Hz ETA*TM_Hz];
            mNaN = isnan(mZ);
            if any(mNaN)
                new_cols = sum(~mNaN(1,:));
                mZ = reshape(mZ(~mNaN), rows, new_cols);
                mBNegate = true(size(mZ));
                mBNegate([rows rows+indEz rows+indHt rows+indHz], new_cols) = 1;
            else
                mBNegate([rows rows+indEz rows+indHt rows+indHz], cols) = 1;
            end
        end
    end
end

%mZ = sparse(mZ);
if nargout == 2
    varargout(1) = {mBNegate};
end