function curves = out2curves(out)
% curves = OUT2CURVES(SMTout)
% Creates a family of dispersion curves from the output of smtgui, i.e., the SMTout
% variable saved to the workspace from the results window.
% the output, curves, is an array which has two fields:
% curves.neff - a vector of effective indices, and
% curves.lambda - a vector of the corresponding wavelengths.
vLambda = [out.lambda];
vNeff = [out.neff];
tol = 1e-10;
vUniLambda = approx_unique(vLambda, tol);
nModes = 0;
for iLambda = 1:length(vUniLambda)
    m = sum(app_eq(vLambda, vUniLambda(iLambda), tol));
    if m > nModes
        nModes = m;
    end
end
mCurves = zeros(nModes, length(vUniLambda));
for iLambda = 1:length(vUniLambda)
    lambda = vUniLambda(iLambda);
    neff = vNeff(app_eq(vLambda, lambda, tol));
    neff = sort(neff, 'descend');
    mCurves(1:length(neff),iLambda) = neff;
end
for iMode = 1:nModes
    neff = mCurves(iMode,:);
    bNonZero = neff~=0;
    curves{iMode}.neff = neff(bNonZero);
    curves{iMode}.lambda = vUniLambda(bNonZero);
end
    

